"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SandpackClient = void 0;
var templates_1 = require("codesandbox-import-utils/lib/create-sandbox/templates");
var lodash_isequal_1 = require("lodash.isequal");
var utils_1 = require("./utils");
var iframe_protocol_1 = require("./iframe-protocol");
// @ts-ignore
var package_json_1 = require("../package.json");
var BUNDLER_URL = process.env.CODESANDBOX_ENV === 'development'
    ? 'http://localhost:3000/'
    : "https://" + package_json_1.version.replace(/\./g, '-') + "-sandpack.codesandbox.io/";
var SandpackClient = /** @class */ (function () {
    function SandpackClient(selector, sandboxInfo, options) {
        var _this = this;
        if (options === void 0) { options = {}; }
        this.getTranspilerContext = function () {
            return new Promise(function (resolve) {
                var unsubscribe = _this.listen(function (message) {
                    if (message.type === 'transpiler-context') {
                        resolve(message.data);
                        unsubscribe();
                    }
                });
                _this.dispatch({ type: 'get-transpiler-context' });
            });
        };
        this.options = options;
        this.sandboxInfo = sandboxInfo;
        this.bundlerURL = options.bundlerURL || BUNDLER_URL;
        this.bundlerState = undefined;
        this.errors = [];
        this.status = 'initializing';
        if (typeof selector === 'string') {
            this.selector = selector;
            var element = document.querySelector(selector);
            if (!element) {
                throw new Error("No element found for selector '" + selector + "'");
            }
            this.element = element;
            this.iframe = document.createElement('iframe');
            this.initializeElement();
        }
        else {
            this.element = selector;
            this.iframe = selector;
        }
        if (!this.iframe.getAttribute('sandbox')) {
            this.iframe.setAttribute('sandbox', 'allow-forms allow-modals allow-popups allow-presentation allow-same-origin allow-scripts');
        }
        if (!this.iframe.getAttribute('allow')) {
            this.iframe.setAttribute('allow', 'accelerometer; ambient-light-sensor; autoplay; camera; encrypted-media; geolocation; gyroscope; hid; microphone; midi; payment; usb; vr; xr-spatial-tracking');
        }
        this.iframe.src = this.bundlerURL;
        this.iframeProtocol = new iframe_protocol_1.IFrameProtocol(this.iframe, this.bundlerURL);
        this.unsubscribeGlobalListener = this.iframeProtocol.globalListen(function (mes) {
            if (mes.type !== 'initialized' ||
                // mes.url !== this.bundlerURL || TODO: see if it makes sense to match the URL here (eg: routing scenario)
                !_this.iframe.contentWindow) {
                return;
            }
            _this.iframeProtocol.register();
            // TODO: Figure out the best way to bring this back
            // if (this.options.fileResolver) {
            //   this.fileResolverProtocol = new Protocol(
            //     'file-resolver',
            //     async (data: { m: 'isFile' | 'readFile'; p: string }) => {
            //       if (data.m === 'isFile') {
            //         return this.options.fileResolver!.isFile(data.p);
            //       }
            //       return this.options.fileResolver!.readFile(data.p);
            //     },
            //     this.iframe.contentWindow
            //   );
            // }
            _this.updatePreview(_this.sandboxInfo, true);
        });
        this.unsubscribeChannelListener = this.iframeProtocol.channelListen(function (mes) {
            switch (mes.type) {
                case 'start': {
                    _this.errors = [];
                    break;
                }
                case 'status': {
                    _this.status = mes.status;
                    break;
                }
                case 'action': {
                    if (mes.action === 'show-error') {
                        var title = mes.title, path = mes.path, message = mes.message, line = mes.line, column = mes.column;
                        _this.errors = __spreadArrays(_this.errors, [
                            { title: title, path: path, message: message, line: line, column: column },
                        ]);
                    }
                    break;
                }
                case 'state': {
                    _this.bundlerState = mes.state;
                    break;
                }
            }
        });
    }
    SandpackClient.prototype.cleanup = function () {
        this.unsubscribeChannelListener();
        this.unsubscribeGlobalListener();
        this.iframeProtocol.cleanup();
    };
    SandpackClient.prototype.updateOptions = function (options) {
        if (!lodash_isequal_1.default(this.options, options)) {
            this.options = options;
            this.updatePreview();
        }
    };
    SandpackClient.prototype.updatePreview = function (sandboxInfo, isInitializationCompile) {
        var _a, _b, _c;
        if (sandboxInfo === void 0) { sandboxInfo = this.sandboxInfo; }
        this.sandboxInfo = sandboxInfo;
        var files = this.getFiles();
        var modules = Object.keys(files).reduce(function (prev, next) {
            var _a;
            return (__assign(__assign({}, prev), (_a = {}, _a[next] = {
                code: files[next].code,
                path: next,
            }, _a)));
        }, {});
        var packageJSON = JSON.parse(utils_1.createPackageJSON(this.sandboxInfo.dependencies, this.sandboxInfo.entry));
        try {
            packageJSON = JSON.parse(files['/package.json'].code);
        }
        catch (e) {
            console.error('Could not parse package.json file: ' + e.message);
        }
        // TODO move this to a common format
        var normalizedModules = Object.keys(files).reduce(function (prev, next) {
            var _a;
            return (__assign(__assign({}, prev), (_a = {}, _a[next] = {
                content: files[next].code,
                path: next,
            }, _a)));
        }, {});
        this.dispatch({
            type: 'compile',
            codesandbox: true,
            version: 3,
            isInitializationCompile: isInitializationCompile,
            modules: modules,
            externalResources: [],
            hasFileResolver: Boolean(this.options.fileResolver),
            disableDependencyPreprocessing: this.sandboxInfo
                .disableDependencyPreprocessing,
            template: this.sandboxInfo.template ||
                templates_1.getTemplate(packageJSON, normalizedModules),
            showOpenInCodeSandbox: (_a = this.options.showOpenInCodeSandbox) !== null && _a !== void 0 ? _a : true,
            showErrorScreen: (_b = this.options.showErrorScreen) !== null && _b !== void 0 ? _b : true,
            showLoadingScreen: (_c = this.options.showLoadingScreen) !== null && _c !== void 0 ? _c : true,
            skipEval: this.options.skipEval || false,
        });
    };
    SandpackClient.prototype.dispatch = function (message) {
        this.iframeProtocol.dispatch(message);
    };
    SandpackClient.prototype.listen = function (listener) {
        return this.iframeProtocol.channelListen(listener);
    };
    /**
     * Get the URL of the contents of the current sandbox
     */
    SandpackClient.prototype.getCodeSandboxURL = function () {
        var files = this.getFiles();
        var paramFiles = Object.keys(files).reduce(function (prev, next) {
            var _a;
            return (__assign(__assign({}, prev), (_a = {}, _a[next.replace('/', '')] = {
                content: files[next].code,
                isBinary: false,
            }, _a)));
        }, {});
        return fetch('https://codesandbox.io/api/v1/sandboxes/define?json=1', {
            method: 'POST',
            body: JSON.stringify({ files: paramFiles }),
            headers: {
                Accept: 'application/json',
                'Content-Type': 'application/json',
            },
        })
            .then(function (x) { return x.json(); })
            .then(function (res) { return ({
            sandboxId: res.sandbox_id,
            editorUrl: "https://codesandbox.io/s/" + res.sandbox_id,
            embedUrl: "https://codesandbox.io/embed/" + res.sandbox_id,
        }); });
    };
    SandpackClient.prototype.getFiles = function () {
        var sandboxInfo = this.sandboxInfo;
        if (sandboxInfo.files['/package.json'] === undefined) {
            return utils_1.addPackageJSONIfNeeded(sandboxInfo.files, sandboxInfo.dependencies, sandboxInfo.entry);
        }
        return this.sandboxInfo.files;
    };
    SandpackClient.prototype.initializeElement = function () {
        this.iframe.style.border = '0';
        this.iframe.style.width = this.options.width || '100%';
        this.iframe.style.height = this.options.height || '100%';
        this.iframe.style.overflow = 'hidden';
        if (!this.element.parentNode) {
            // This should never happen
            throw new Error('Given element does not have a parent.');
        }
        this.element.parentNode.replaceChild(this.iframe, this.element);
    };
    return SandpackClient;
}());
exports.SandpackClient = SandpackClient;
//# sourceMappingURL=client.js.map